import fetch from "node-fetch";
function escapeRegex(string) {
    return string.replace(/[/\-\\^$*+?.()|[\]{}]/g, '\\$&');
}
/**
 * This function attempts to parse a Response as ResponseData.
 */
async function parseResponse(response) {
    let text = null;
    try {
        text = await response.text();
        const json = JSON.parse(text);
        return { status: response.status, body: json };
    }
    catch {
        return { status: response.status, body: text };
    }
}
export class Client {
    BASE_URL;
    SERVICE_URL;
    fixedHeaders;
    constructor(BASE_URL, SERVICE_URL, fixedHeaders) {
        this.BASE_URL = BASE_URL;
        this.SERVICE_URL = SERVICE_URL ?? BASE_URL;
        this.fixedHeaders = fixedHeaders ?? {};
    }
    fetch = async (url, init) => {
        let raw_response;
        init.headers = init.headers || { 'Content-Type': 'application/json' };
        init.headers = { ...this.fixedHeaders, ...init.headers, };
        try {
            raw_response = await fetch(url, init);
        }
        catch (error) {
            if (error instanceof Error) {
                throw new Error(error.message);
            }
            else if (typeof error === 'string') {
                throw new Error(error);
            }
            else {
                throw new Error('Something went wrong while trying to fetch the request');
            }
        }
        return await parseResponse(raw_response);
    };
    /*
    Authenticate user
    
    This endpoint will authenticate a user and return a JWT for the internal use of the microservice architecture.
    The return code will always be 200, but the JWT will be empty if the authentication failed.
    */
    async auth(url = "/auth", authorization, xRealIP, xForwardedProto) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(auth)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/auth';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        if (authorization) {
            url_with_query_params.searchParams.set('authorization', authorization);
        }
        if (xRealIP) {
            url_with_query_params.searchParams.set('xRealIP', xRealIP);
        }
        if (xForwardedProto) {
            url_with_query_params.searchParams.set('xForwardedProto', xForwardedProto);
        }
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Login user
    
    This endpoint will login a user and return an access token for the use of the microservice architecture.
    */
    async login(body, url = "/login") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(login)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/login';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 201) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Logout user
    
    This endpoint will logout a user and remove the corresponding access token for the use of the microservice architecture.
    */
    async logout(body, url = "/logout") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(logout)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/logout';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 204) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Get all users
    
    */
    async listUsers(url = "/users") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(users)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/users';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Create a user
    
    */
    async createUser(body, url = "/users") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(users)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/users';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 201) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Get a user
    
    */
    async getUser(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(users\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Update a user
    
    */
    async updateUser(url, body) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(users\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "patch", body: JSON.stringify(body) });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Delete a user
    
    */
    async deleteUser(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(users\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "delete" });
        // transform response
        if (response.status === 204) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Get identity
    
    */
    async getIdentity(url = "/identity") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(identity)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/identity';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Update identity
    
    */
    async updateIdentity(url = "/identity", body) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(identity)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/identity';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "patch", body: JSON.stringify(body) });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Create a new token
    
    This endpoint will create a new token for the use of the microservice architecture.
    */
    async createToken(body, url = "/token") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(token)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/token';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 201) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
}
//# sourceMappingURL=client.js.map