import fetch from "node-fetch";
function escapeRegex(string) {
    return string.replace(/[/\-\\^$*+?.()|[\]{}]/g, '\\$&');
}
/**
 * This function attempts to parse a Response as ResponseData.
 */
async function parseResponse(response) {
    let text = null;
    try {
        text = await response.text();
        const json = JSON.parse(text);
        return { status: response.status, body: json };
    }
    catch {
        return { status: response.status, body: text };
    }
}
export class Client {
    BASE_URL;
    SERVICE_URL;
    fixedHeaders;
    constructor(BASE_URL, SERVICE_URL, fixedHeaders) {
        this.BASE_URL = BASE_URL;
        this.SERVICE_URL = SERVICE_URL ?? BASE_URL;
        this.fixedHeaders = fixedHeaders ?? {};
    }
    fetch = async (url, init) => {
        let raw_response;
        init.headers = init.headers || { 'Content-Type': 'application/json' };
        init.headers = { ...this.fixedHeaders, ...init.headers, };
        try {
            raw_response = await fetch(url, init);
        }
        catch (error) {
            if (error instanceof Error) {
                throw new Error(error.message);
            }
            else if (typeof error === 'string') {
                throw new Error(error);
            }
            else {
                throw new Error('Something went wrong while trying to fetch the request');
            }
        }
        return await parseResponse(raw_response);
    };
    /*
    List experiments
    
    */
    async listExperiments(url = "/experiments") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(experiments)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/experiments';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Create a new experiment
    
    */
    async createExperiment(body, url = "/experiments") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(experiments)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/experiments';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 201) {
            return response.body;
        }
        if (response.status === 202) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    View an experiment.
    
    */
    async getExperiment(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(experiments\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Update an existing experiment.
    
    With this endpoint an experiment can be changed. The request body may be skipped if you just want to set a hook via the query string parameters.
    
    If a body is supplied you can choose to include any first level fields which will fully replace the field in the existing experiment.
    
    */
    async updateExperiment(url, body, changedURL) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(experiments\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        if (changedURL) {
            url_with_query_params.searchParams.set('changedURL', changedURL);
        }
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "patch", body: JSON.stringify(body) });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        if (response.status === 202) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Delete an experiment
    
    */
    async deleteExperiment(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(experiments\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "delete" });
        // transform response
        if (response.status === 204) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    List templates
    
    */
    async listTemplate(url = "/templates") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(templates)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/templates';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Create a new template
    
    */
    async createTemplate(body, url = "/templates") {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?()(templates)?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '/templates';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "post", body: JSON.stringify(body) });
        // transform response
        if (response.status === 201) {
            return response.body;
        }
        if (response.status === 202) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    View an template.
    
    */
    async getTemplate(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(templates\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "get" });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Update an existing template.
    
    With this endpoint an template can be changed. The request body may be skipped if you just want to set a hook via the query string parameters.
    
    If a body is supplied you can choose to include any first level fields which will fully replace the field in the existing template.
    
    */
    async updateTemplate(url, body) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(templates\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "patch", body: JSON.stringify(body) });
        // transform response
        if (response.status === 200) {
            return response.body;
        }
        if (response.status === 202) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
    /*
    Delete an template
    
    */
    async deleteTemplate(url) {
        // match path to url schema
        const m = new RegExp('^(' + escapeRegex(this.BASE_URL) + ')?\/?(templates\/[^?]*?)()?$').exec(url);
        if (!m)
            throw new Error("Invalid url");
        let valid_url = '/' + m[2] + '';
        if (valid_url.startsWith('//'))
            valid_url = valid_url.slice(1);
        const url_with_query_params = new URL(this.SERVICE_URL + valid_url, this.SERVICE_URL);
        // make http call
        const response = await this.fetch(url_with_query_params, { method: "delete" });
        // transform response
        if (response.status === 204) {
            return response.body;
        }
        throw new Error("Unexpected status code: " + response.status);
    }
}
//# sourceMappingURL=client.js.map